@@ -124,6 +124,9 @@ pub(crate) enum ClientError {
     /// We couldn't turn the user's token into a valid header value.
     #[error("invalid token header")]
     InvalidTokenHeader(#[from] InvalidHeaderValue),
+    /// An error originating from encoding or decoding Git pkt-lines.
+    #[error("error while processing Git pkt-lines")]
+    PktLint(#[from] pktline::PktLineError),
     /// An error originating from listing refs through direct
     /// Git access.
     #[error("error while listing Git references")]
@@ -180,13 +183,17 @@ impl reqwest_middleware::Middleware for CacheLoggingMiddleware {
     ) -> reqwest_middleware::Result<Response> {
         tracing::debug!("Request URL: {}", req.url());
 
-        let res = next.run(req, extensions).await;
+        let res = next.run(req, extensions).await?;
 
-        let cache_type = extensions.get::<CacheType>().unwrap();
-        let cache_result = extensions.get::<CacheResult>().unwrap();
+        let cache_type = extensions
+            .get::<CacheType>()
+            .expect("internal error: expected CacheType");
+        let cache_result = extensions
+            .get::<CacheResult>()
+            .expect("internal error: expected CacheResult");
         tracing::debug!("{:?} cache was {:?}", cache_type, cache_result);
 
-        res
+        Ok(res)
     }
 }
 
@@ -207,7 +214,10 @@ impl<T: CacheManager> reqwest_middleware::Middleware for ChainedCache<T> {
         if let Ok(ref resp) = res
             && let Some(cache) = resp.headers().get("x-cache")
         {
-            let cache_result = match cache.to_str().unwrap() {
+            let cache_result = match cache
+                .to_str()
+                .expect("invalid x-cache header (not a string)")
+            {
                 "HIT" => CacheResult::Hit,
                 _ => CacheResult::Miss,
             };
@@ -251,8 +261,8 @@ impl Client {
         // GitHub REST API client.
         let mut api_client_headers = HeaderMap::new();
         api_client_headers.insert(AUTHORIZATION, token.to_header_value()?);
-        api_client_headers.insert("X-GitHub-Api-Version", "2022-11-28".parse().unwrap());
-        api_client_headers.insert(ACCEPT, "application/vnd.github+json".parse().unwrap());
+        api_client_headers.insert("X-GitHub-Api-Version", "2022-11-28".parse()?);
+        api_client_headers.insert(ACCEPT, "application/vnd.github+json".parse()?);
 
         let api_client = Self::default_middleware(
             cache_dir,
@@ -345,28 +355,14 @@ impl Client {
                 // We additionally use the ref-prefix arguments to (hopefully) limit
                 // the server's response to only branches and tags.
                 let mut req = vec![];
-                pktline::Packet::data("command=ls-refs\n".as_bytes())
-                    .unwrap()
-                    .encode(&mut req)
-                    .unwrap();
-                pktline::Packet::data(format!("agent={}\n", ZIZMOR_AGENT).as_bytes())
-                    .unwrap()
-                    .encode(&mut req)
-                    .unwrap();
-                pktline::Packet::Delim.encode(&mut req).unwrap();
-                pktline::Packet::data("peel\n".as_bytes())
-                    .unwrap()
-                    .encode(&mut req)
-                    .unwrap();
-                pktline::Packet::data("ref-prefix refs/heads/\n".as_bytes())
-                    .unwrap()
-                    .encode(&mut req)
-                    .unwrap();
-                pktline::Packet::data("ref-prefix refs/tags/\n".as_bytes())
-                    .unwrap()
-                    .encode(&mut req)
-                    .unwrap();
-                pktline::Packet::Flush.encode(&mut req).unwrap();
+                pktline::Packet::data("command=ls-refs\n".as_bytes())?.encode(&mut req)?;
+                pktline::Packet::data(format!("agent={}\n", ZIZMOR_AGENT).as_bytes())?
+                    .encode(&mut req)?;
+                pktline::Packet::Delim.encode(&mut req)?;
+                pktline::Packet::data("peel\n".as_bytes())?.encode(&mut req)?;
+                pktline::Packet::data("ref-prefix refs/heads/\n".as_bytes())?.encode(&mut req)?;
+                pktline::Packet::data("ref-prefix refs/tags/\n".as_bytes())?.encode(&mut req)?;
+                pktline::Packet::Flush.encode(&mut req)?;
 
                 let resp = self
                     .base_client
@@ -377,17 +373,19 @@ impl Client {
                     .send()
                     .await?;
 
-                let resp = match resp.status() {
-                    StatusCode::OK => Ok(resp),
+                let resp = match resp.error_for_status() {
+                    Ok(resp) => Ok(resp),
                     // NOTE: Versions of zizmor prior to 1.16.0 would silently
                     // skip private or missing repositories, as branch/tag lookups
                     // were done as a binary present/absent check. This caused
                     // false negatives.
-                    StatusCode::NOT_FOUND => Err(ClientError::RepoMissingOrPrivate {
-                        owner: owner.to_string(),
-                        repo: repo.to_string(),
-                    }),
-                    _ => Err(resp.error_for_status().unwrap_err().into()),
+                    Err(e) if e.status() == Some(StatusCode::NOT_FOUND) => {
+                        Err(ClientError::RepoMissingOrPrivate {
+                            owner: owner.to_string(),
+                            repo: repo.to_string(),
+                        })
+                    }
+                    Err(e) => Err(e.into()),
                 }?;
 
                 let mut remote_refs = vec![];
@@ -444,7 +442,6 @@ impl Client {
     }
 
     #[instrument(skip(self))]
-    #[tokio::main]
     pub(crate) async fn list_branches(
         &self,
         owner: &str,
@@ -495,13 +492,11 @@ impl Client {
     }
 
     #[instrument(skip(self))]
-    #[tokio::main]
     pub(crate) async fn list_tags(&self, owner: &str, repo: &str) -> Result<Vec<Tag>, ClientError> {
         self.list_tags_internal(owner, repo).await
     }
 
     #[instrument(skip(self))]
-    #[tokio::main]
     pub(crate) async fn has_branch(
         &self,
         owner: &str,
@@ -516,7 +511,6 @@ impl Client {
     }
 
     #[instrument(skip(self))]
-    #[tokio::main]
     pub(crate) async fn has_tag(
         &self,
         owner: &str,
@@ -531,7 +525,6 @@ impl Client {
     }
 
     #[instrument(skip(self))]
-    #[tokio::main]
     pub(crate) async fn commit_for_ref(
         &self,
         owner: &str,
@@ -560,7 +553,7 @@ impl Client {
     }
 
     #[instrument(skip(self))]
-    pub(crate) fn longest_tag_for_commit(
+    pub(crate) async fn longest_tag_for_commit(
         &self,
         owner: &str,
         repo: &str,
@@ -571,7 +564,7 @@ impl Client {
         // do it for them.
         // This could be optimized in various ways, not least of which
         // is not pulling every tag eagerly before scanning them.
-        let tags = self.list_tags(owner, repo)?;
+        let tags = self.list_tags(owner, repo).await?;
 
         // Heuristic: there can be multiple tags for a commit, so we pick
         // the longest one. This isn't super sound, but it gets us from
@@ -583,7 +576,29 @@ impl Client {
     }
 
     #[instrument(skip(self))]
-    #[tokio::main]
+    pub(crate) async fn branch_commits(
+        &self,
+        owner: &str,
+        repo: &str,
+        commit: &str,
+    ) -> Result<BranchCommits, ClientError> {
+        // NOTE(ww): This API is undocumented.
+        // See: https://github.com/orgs/community/discussions/78161
+        let url = format!("https://github.com/{owner}/{repo}/branch_commits/{commit}");
+
+        // We ask GitHub for JSON, because it sends HTML by default for this endpoint.
+        self.base_client
+            .get(&url)
+            .header(ACCEPT, "application/json")
+            .send()
+            .await?
+            .error_for_status()?
+            .json()
+            .await
+            .map_err(Into::into)
+    }
+
+    #[instrument(skip(self))]
     pub(crate) async fn compare_commits(
         &self,
         owner: &str,
@@ -598,18 +613,17 @@ impl Client {
 
         let resp = self.api_client.get(url).send().await?;
 
-        match resp.status() {
-            StatusCode::OK => {
-                Ok::<_, reqwest::Error>(Some(resp.json::<Comparison>().await?.status))
+        match resp.error_for_status() {
+            Ok(resp) => {
+                let comparison: Comparison = resp.json().await?;
+                Ok(Some(comparison.status))
             }
-            StatusCode::NOT_FOUND => Ok(None),
-            _ => Err(resp.error_for_status().unwrap_err()),
+            Err(e) if e.status() == Some(StatusCode::NOT_FOUND) => Ok(None),
+            Err(e) => Err(e.into()),
         }
-        .map_err(Into::into)
     }
 
     #[instrument(skip(self))]
-    #[tokio::main]
     pub(crate) async fn gha_advisories(
         &self,
         owner: &str,
@@ -638,7 +652,6 @@ impl Client {
     /// Returns the file contents as a `String` if the file exists,
     /// or `None` if the request produces a 404.
     #[instrument(skip(self, slug, file))]
-    #[tokio::main]
     pub(crate) async fn fetch_single_file(
         &self,
         slug: &RepoSlug,
@@ -673,14 +686,14 @@ impl Client {
             .send()
             .await?;
 
-        match resp.status() {
-            StatusCode::OK => {
+        match resp.error_for_status() {
+            Ok(resp) => {
                 let contents = resp.text().await?;
 
                 Ok(Some(contents))
             }
-            StatusCode::NOT_FOUND => Ok(None),
-            _ => Err(resp.error_for_status().unwrap_err().into()),
+            Err(e) if e.status() == Some(StatusCode::NOT_FOUND) => Ok(None),
+            Err(e) => Err(e.into()),
         }
     }
 
@@ -690,7 +703,6 @@ impl Client {
     /// This is an optimized variant of `fetch_audit_inputs` for the workflow-only
     /// collection case.
     #[instrument(skip(self, options, group))]
-    #[tokio::main]
     pub(crate) async fn fetch_workflows(
         &self,
         slug: &RepoSlug,
@@ -722,7 +734,13 @@ impl Client {
             .await
             .map_err(ClientError::from)?
             .error_for_status()
-            .map_err(ClientError::from)?
+            .map_err(|err| match err {
+                // TODO: Disambiguate a 404 from missing repo vs missing workflows dir.
+                e if e.status() == Some(StatusCode::NOT_FOUND) => {
+                    CollectionError::RemoteWithoutWorkflows(ClientError::from(e), slug.to_string())
+                }
+                e => ClientError::from(e).into(),
+            })?
             .json()
             .await
             .map_err(ClientError::from)?;
@@ -742,7 +760,7 @@ impl Client {
                 .into());
             };
 
-            let key = InputKey::remote(slug, file.path)?;
+            let key = InputKey::remote(slug, file.path);
             group.register(InputKind::Workflow, contents, key, options.strict)?;
         }
 
@@ -755,7 +773,6 @@ impl Client {
     /// This is much slower than `fetch_workflows`, since it involves
     /// retrieving the entire repository archive and decompressing it.
     #[instrument(skip(self, options, group))]
-    #[tokio::main]
     pub(crate) async fn fetch_audit_inputs(
         &self,
         slug: &RepoSlug,
@@ -813,12 +830,12 @@ impl Client {
                     .parent()
                     .is_some_and(|dir| dir.ends_with(".github/workflows"))
             {
-                let key = InputKey::remote(slug, file_path.to_string())?;
+                let key = InputKey::remote(slug, file_path.to_string());
                 let mut contents = String::with_capacity(entry.size() as usize);
                 entry.read_to_string(&mut contents)?;
                 group.register(InputKind::Workflow, contents, key, options.strict)?;
             } else if matches!(file_path.file_name(), Some("action.yml" | "action.yaml")) {
-                let key = InputKey::remote(slug, file_path.to_string())?;
+                let key = InputKey::remote(slug, file_path.to_string());
                 let mut contents = String::with_capacity(entry.size() as usize);
                 entry.read_to_string(&mut contents)?;
                 group.register(InputKind::Action, contents, key, options.strict)?;
@@ -826,7 +843,7 @@ impl Client {
                 file_path.file_name(),
                 Some("dependabot.yml" | "dependabot.yaml")
             ) {
-                let key = InputKey::remote(slug, file_path.to_string())?;
+                let key = InputKey::remote(slug, file_path.to_string());
                 let mut contents = String::with_capacity(entry.size() as usize);
                 entry.read_to_string(&mut contents)?;
                 group.register(InputKind::Dependabot, contents, key, options.strict)?;
@@ -865,6 +882,23 @@ pub(crate) struct Commit {
     pub(crate) sha: String,
 }
 
+/// The response structure from GitHub's undocumented `branch_commits` API.
+///
+/// This model is intentionally incomplete.
+#[derive(Clone, Deserialize)]
+#[serde(rename_all = "lowercase")]
+#[non_exhaustive]
+pub(crate) struct BranchCommits {
+    branches: Vec<serde_json::Value>,
+    tags: Vec<String>,
+}
+
+impl BranchCommits {
+    pub(crate) fn is_empty(&self) -> bool {
+        self.branches.is_empty() && self.tags.is_empty()
+    }
+}
+
 #[derive(Clone, Deserialize)]
 #[serde(rename_all = "lowercase")]
 pub(crate) enum ComparisonStatus {
