@@ -11,14 +11,14 @@ fn test_pedantic_persona() -> anyhow::Result<()> {
             .run()?,
         @r"
     help[stale-action-refs]: commit hash does not point to a Git tag
-      --> @@INPUT@@:34:13
+      --> @@INPUT@@:34:15
        |
-    34 |     - uses: actions/checkout@009b9ae9e446ad8d9b8c809870b0fbcc5e03573e
-       |             ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ this step
+    34 |       - uses: actions/checkout@009b9ae9e446ad8d9b8c809870b0fbcc5e03573e
+       |               ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ this step
        |
        = note: audit confidence → High
 
-    1 finding: 0 informational, 1 low, 0 medium, 0 high
+    4 findings (3 ignored): 0 informational, 1 low, 0 medium, 0 high
     "
     );
 
